@@ -32,8 +32,6 @@ extern rpc_clnt_prog_t *cli_rpc_prog;
 int
 cli_cmd_global_help_cbk (struct cli_state *state, struct cli_cmd_word *in_word,
                          const char **words, int wordcount);
-int cli_cmd_ganesha_cbk (struct cli_state *state, struct cli_cmd_word *word,
-                                         const char **words, int wordcount);
 int
 cli_cmd_get_state_cbk (struct cli_state *state, struct cli_cmd_word *word,
                               const char **words, int wordcount);
@@ -43,11 +41,8 @@ struct cli_cmd global_cmds[] = {
            cli_cmd_global_help_cbk,
            "list global commands",
         },
-        { "nfs-ganesha {enable| disable} ",
-           cli_cmd_ganesha_cbk,
-          "Enable/disable NFS-Ganesha support",
-        },
-        { "get-state [<daemon>] [odir </path/to/output/dir/>] [file <filename>]",
+        { "get-state [<daemon>] [[odir </path/to/output/dir/>] "
+          "[file <filename>]] [detail]",
           cli_cmd_get_state_cbk,
           "Get local state representation of mentioned daemon",
         },
@@ -91,54 +86,6 @@ out:
 
 }
 
-int cli_cmd_ganesha_cbk (struct cli_state *state, struct cli_cmd_word *word,
-                         const char **words, int wordcount)
-
-{
-         int                     sent        =   0;
-         int                     parse_error =   0;
-         int                     ret         =  -1;
-         rpc_clnt_procedure_t    *proc       =  NULL;
-         call_frame_t            *frame      =  NULL;
-         dict_t                  *options    =  NULL;
-         cli_local_t             *local      =  NULL;
-         char                    *op_errstr  =  NULL;
-
-        proc = &cli_rpc_prog->proctable[GLUSTER_CLI_GANESHA];
-
-        frame = create_frame (THIS, THIS->ctx->pool);
-        if (!frame)
-                goto out;
-
-        ret = cli_cmd_ganesha_parse (state, words, wordcount,
-                                     &options, &op_errstr);
-        if (ret) {
-                if (op_errstr) {
-                    cli_err ("%s", op_errstr);
-                    GF_FREE (op_errstr);
-                } else
-                    cli_usage_out (word->pattern);
-                parse_error = 1;
-                goto out;
-        }
-
-        CLI_LOCAL_INIT (local, words, frame, options);
-
-        if (proc->fn) {
-                ret = proc->fn (frame, THIS, options);
-        }
-
-out:
-        if (ret) {
-                cli_cmd_sent_status_get (&sent);
-                if ((sent == 0) && (parse_error == 0))
-                        cli_out ("Setting global option failed");
-        }
-
-        CLI_STACK_DESTROY (frame);
-        return ret;
-}
-
 int
 cli_cmd_get_state_cbk (struct cli_state *state, struct cli_cmd_word *word,
                        const char **words, int wordcount)
