@@ -153,22 +153,11 @@ unlock:
 inode_t *
 afr_shd_inode_find (xlator_t *this, xlator_t *subvol, uuid_t gfid)
 {
-	int           ret      = 0;
-        uint64_t       val     = IA_INVAL;
-	loc_t         loc      = {0, };
+        int          ret       = 0;
+        uint64_t     val       = IA_INVAL;
         dict_t       *xdata    = NULL;
         dict_t       *rsp_dict = NULL;
-	inode_t      *inode    = NULL;
-	struct iatt   iatt     = {0, };
-
-	inode = inode_find (this->itable, gfid);
-	if (inode)
-		goto out;
-
-	loc.inode = inode_new (this->itable);
-	if (!loc.inode)
-		goto out;
-	gf_uuid_copy (loc.gfid, gfid);
+        inode_t      *inode    = NULL;
 
         xdata = dict_new ();
         if (!xdata)
@@ -178,7 +167,8 @@ afr_shd_inode_find (xlator_t *this, xlator_t *subvol, uuid_t gfid)
         if (ret)
                 goto out;
 
-	ret = syncop_lookup (subvol, &loc, &iatt, NULL, xdata, &rsp_dict);
+	ret = syncop_inode_find (this, subvol, gfid, &inode,
+                                 xdata, &rsp_dict);
 	if (ret < 0)
 		goto out;
 
@@ -188,15 +178,16 @@ afr_shd_inode_find (xlator_t *this, xlator_t *subvol, uuid_t gfid)
                 if (ret)
                         goto out;
         }
-
-	inode = inode_link (loc.inode, NULL, NULL, &iatt);
         ret = inode_ctx_set2 (inode, subvol, 0, &val);
 out:
+        if (ret && inode) {
+                inode_unref (inode);
+                inode = NULL;
+        }
         if (xdata)
                 dict_unref (xdata);
         if (rsp_dict)
                 dict_unref (rsp_dict);
-	loc_wipe (&loc);
 	return inode;
 }
 
@@ -428,8 +419,8 @@ afr_shd_index_heal (xlator_t *subvol, gf_dirent_t *entry, loc_t *parent,
         if (!priv->shd.enabled)
                 return -EBUSY;
 
-        gf_msg_debug (healer->this->name, 0, "got entry: %s",
-                      entry->d_name);
+        gf_msg_debug (healer->this->name, 0, "got entry: %s from %s",
+                      entry->d_name, priv->children[healer->subvol]->name);
 
         ret = gf_uuid_parse (entry->d_name, gfid);
         if (ret)
@@ -773,7 +764,7 @@ afr_shd_dict_add_crawl_event (xlator_t *this, dict_t *output,
         if (ret) {
                 gf_msg (this->name, GF_LOG_ERROR,
                         -ret, AFR_MSG_DICT_SET_FAILED,
-		        "Could not add statistics_healed_count to outout");
+		        "Could not add statistics_healed_count to output");
                 goto out;
 	}
 
@@ -783,7 +774,7 @@ afr_shd_dict_add_crawl_event (xlator_t *this, dict_t *output,
 	if (ret) {
                 gf_msg (this->name, GF_LOG_ERROR,
                         -ret, AFR_MSG_DICT_SET_FAILED,
-		        "Could not add statistics_split_brain_count to outout");
+		        "Could not add statistics_split_brain_count to output");
                 goto out;
         }
 
@@ -803,7 +794,7 @@ afr_shd_dict_add_crawl_event (xlator_t *this, dict_t *output,
 	if (ret) {
                 gf_msg (this->name, GF_LOG_ERROR,
                         -ret, AFR_MSG_DICT_SET_FAILED,
-	                "Could not add statistics_healed_failed_count to outout");
+	                "Could not add statistics_healed_failed_count to output");
                 goto out;
         }
 
@@ -813,7 +804,7 @@ afr_shd_dict_add_crawl_event (xlator_t *this, dict_t *output,
 	if (ret) {
                 gf_msg (this->name, GF_LOG_ERROR,
                         -ret, AFR_MSG_DICT_SET_FAILED,
-		        "Could not add statistics_crawl_start_time to outout");
+		        "Could not add statistics_crawl_start_time to output");
                 goto out;
         } else {
 		start_time_str = NULL;
@@ -832,7 +823,7 @@ afr_shd_dict_add_crawl_event (xlator_t *this, dict_t *output,
 	if (ret) {
                 gf_msg (this->name, GF_LOG_ERROR,
                         -ret, AFR_MSG_DICT_SET_FAILED,
-		        "Could not add statistics_crawl_end_time to outout");
+		        "Could not add statistics_crawl_end_time to output");
                 goto out;
         } else {
 		end_time_str = NULL;
@@ -845,7 +836,7 @@ afr_shd_dict_add_crawl_event (xlator_t *this, dict_t *output,
 	if (ret) {
                 gf_msg (this->name, GF_LOG_ERROR,
                         -ret, AFR_MSG_DICT_SET_FAILED,
-		        "Could not add statistics_inprogress to outout");
+		        "Could not add statistics_inprogress to output");
                 goto out;
         }
 
