@@ -74,6 +74,11 @@ typedef enum {
         AFR_FAV_CHILD_POLICY_MAX,
 } afr_favorite_child_policy;
 
+struct afr_nfsd {
+        gf_boolean_t     iamnfsd;
+        uint32_t         halo_max_latency_msec;
+};
+
 typedef struct _afr_private {
         gf_lock_t lock;               /* to guard access to child_count, etc */
         unsigned int child_count;     /* total number of children   */
@@ -85,6 +90,7 @@ typedef struct _afr_private {
         inode_t *root_inode;
 
         unsigned char *child_up;
+        int64_t *child_latency;
         unsigned char *local;
 
         char **pending_key;
@@ -155,8 +161,14 @@ typedef struct _afr_private {
         gf_boolean_t           ensure_durability;
         char                   *sh_domain;
 	char                   *afr_dirty;
+        gf_boolean_t           halo_enabled;
 
-	afr_self_heald_t       shd;
+        uint32_t               halo_max_latency_msec;
+        uint32_t               halo_max_replicas;
+        uint32_t               halo_min_replicas;
+
+        afr_self_heald_t       shd;
+        struct afr_nfsd        nfsd;
 
         gf_boolean_t           consistent_metadata;
         uint64_t               spb_choice_timeout;
@@ -863,11 +875,6 @@ typedef struct afr_granular_esh_args {
                                   mismatch */
 } afr_granular_esh_args_t;
 
-/* did a call fail due to a child failing? */
-#define child_went_down(op_ret, op_errno) (((op_ret) < 0) &&            \
-                                           ((op_errno == ENOTCONN) ||   \
-                                            (op_errno == EBADFD)))
-
 int
 afr_inode_get_readable (call_frame_t *frame, inode_t *inode, xlator_t *this,
                         unsigned char *readable, int *event_p, int type);
