@@ -30,9 +30,13 @@
 #define GF_DHT_LOOKUP_UNHASHED_AUTO     2
 #define DHT_PATHINFO_HEADER             "DISTRIBUTE:"
 #define DHT_FILE_MIGRATE_DOMAIN         "dht.file.migrate"
+/* Layout synchronization */
 #define DHT_LAYOUT_HEAL_DOMAIN          "dht.layout.heal"
+/* Namespace synchronization */
+#define DHT_ENTRY_SYNC_DOMAIN           "dht.entry.sync"
 #define TIERING_MIGRATION_KEY           "tiering.migration"
 #define DHT_LAYOUT_HASH_INVALID         1
+#define MAX_REBAL_THREADS               sysconf(_SC_NPROCESSORS_ONLN)
 
 #define DHT_DIR_STAT_BLOCKS          8
 #define DHT_DIR_STAT_SIZE            4096
@@ -113,6 +117,11 @@ typedef enum {
         DHT_HASH_TYPE_DM_USER,
 } dht_hashfn_type_t;
 
+typedef enum {
+        DHT_INODELK,
+        DHT_ENTRYLK,
+} dht_lock_type_t;
+
 /* rebalance related */
 struct dht_rebalance_ {
         xlator_t            *from_subvol;
@@ -166,10 +175,52 @@ typedef struct {
         char         *domain;  /* Only locks within a single domain
                                 * contend with each other
                                 */
+        char         *basename; /* Required for entrylk */
         gf_lkowner_t  lk_owner;
         gf_boolean_t  locked;
 } dht_lock_t;
 
+/* The lock structure represents inodelk. */
+typedef struct {
+        fop_inodelk_cbk_t   inodelk_cbk;
+        dht_lock_t        **locks;
+        int                 lk_count;
+        dht_reaction_type_t reaction;
+
+        /* whether locking failed on _any_ of the "locks" above */
+        int                 op_ret;
+        int                 op_errno;
+} dht_ilock_wrap_t;
+
+/* The lock structure represents entrylk. */
+typedef struct {
+        fop_entrylk_cbk_t   entrylk_cbk;
+        dht_lock_t        **locks;
+        int                 lk_count;
+        dht_reaction_type_t reaction;
+
+        /* whether locking failed on _any_ of the "locks" above */
+        int                 op_ret;
+        int                 op_errno;
+} dht_elock_wrap_t;
+
+/* The first member of dht_dir_transaction_t should be of type dht_ilock_wrap_t.
+ * Otherwise it can result in subtle memory corruption issues as in most of the
+ * places we use lock[0].layout.my_layout or lock[0].layout.parent_layout and
+ * lock[0].ns.parent_layout (like in dht_local_wipe).
+ */
+typedef union {
+        union {
+                 dht_ilock_wrap_t my_layout;
+                 dht_ilock_wrap_t parent_layout;
+        } layout;
+        struct dht_namespace {
+                dht_ilock_wrap_t parent_layout;
+                dht_elock_wrap_t directory_ns;
+                fop_entrylk_cbk_t ns_cbk;
+        } ns;
+} dht_dir_transaction_t;
+
 typedef
 int (*dht_selfheal_layout_t)(call_frame_t *frame, loc_t *loc,
                              dht_layout_t *layout);
@@ -288,22 +339,17 @@ struct dht_local {
 
         struct dht_skip_linkto_unlink  skip_unlink;
 
-        struct {
-                fop_inodelk_cbk_t   inodelk_cbk;
-                dht_lock_t        **locks;
-                int                 lk_count;
-                dht_reaction_type_t reaction;
-
-                /* whether locking failed on _any_ of the "locks" above */
-                int                 op_ret;
-                int                 op_errno;
-        } lock;
+        dht_dir_transaction_t lock[2], *current;
 
         short           lock_type;
 
         call_stub_t *stub;
         int32_t      parent_disk_layout[4];
 
+        /* To hold dentries of readdir spawning across subvols */
+        gf_dirent_t   entries;
+        size_t        filled;
+
         /* rename rollback */
         int    *ret_cache ;
 };
@@ -316,10 +362,14 @@ struct dht_du {
         uint64_t avail_space;
         uint32_t log;
         uint32_t chunks;
+        uint32_t total_blocks;
+        uint32_t avail_blocks;
+        uint32_t frsize; /*fragment size*/
 };
 typedef struct dht_du dht_du_t;
 
 enum gf_defrag_type {
+        GF_DEFRAG_CMD_NONE = 0,
         GF_DEFRAG_CMD_START = 1,
         GF_DEFRAG_CMD_STOP = 1 + 1,
         GF_DEFRAG_CMD_STATUS = 1 + 2,
@@ -457,6 +507,7 @@ struct gf_defrag_info_ {
         uint64_t                     num_files_lookedup;
         uint64_t                     total_failures;
         uint64_t                     skipped;
+        uint64_t                     num_dirs_processed;
         gf_lock_t                    lock;
         int                          cmd;
         pthread_t                    th;
@@ -577,7 +628,7 @@ struct dht_conf {
         /* Support size-weighted rebalancing (heterogeneous bricks). */
         gf_boolean_t    do_weighting;
         gf_boolean_t    randomize_by_gfid;
-        char           *dthrottle;
+        int             dthrottle;
 
         dht_methods_t   methods;
 
@@ -1103,7 +1154,7 @@ int dht_newfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                      struct iatt *postparent, dict_t *xdata);
 
 int
-gf_defrag_status_get (gf_defrag_info_t *defrag, dict_t *dict);
+gf_defrag_status_get (dht_conf_t *conf, dict_t *dict);
 
 void
 gf_defrag_set_pause_state (gf_tier_conf_t *tier_conf, tier_pause_state_t state);
@@ -1124,7 +1175,7 @@ int
 gf_defrag_start_detach_tier (gf_defrag_info_t *defrag);
 
 int
-gf_defrag_stop (gf_defrag_info_t *defrag, gf_defrag_status_t status,
+gf_defrag_stop (dht_conf_t *conf, gf_defrag_status_t status,
                 dict_t *output);
 
 void*
@@ -1132,10 +1183,10 @@ gf_defrag_start (void *this);
 
 int32_t
 gf_defrag_handle_hardlink (xlator_t *this, loc_t *loc, dict_t  *xattrs,
-                           struct iatt *stbuf);
+                           struct iatt *stbuf, int *fop_errno);
 int
 dht_migrate_file (xlator_t *this, loc_t *loc, xlator_t *from, xlator_t *to,
-                 int flag);
+                 int flag, int *fop_errno);
 int
 dht_inode_ctx_layout_get (inode_t *inode, xlator_t *this,
                           dht_layout_t **layout_int);
@@ -1158,8 +1209,8 @@ dht_dir_has_layout (dict_t *xattr, char *name);
 gf_boolean_t
 dht_is_subvol_in_layout (dht_layout_t *layout, xlator_t *xlator);
 xlator_t *
-dht_subvol_with_free_space_inodes (xlator_t *this, xlator_t *subvol,
-                                   dht_layout_t *layout);
+dht_subvol_with_free_space_inodes (xlator_t *this, xlator_t *subvol, xlator_t *ignore,
+                                   dht_layout_t *layout, uint64_t filesize);
 xlator_t *
 dht_subvol_maxspace_nonzeroinode (xlator_t *this, xlator_t *subvol,
                                   dht_layout_t *layout);
@@ -1192,47 +1243,6 @@ dht_lookup_everywhere_done (call_frame_t *frame, xlator_t *this);
 int
 dht_fill_dict_to_avoid_unlink_of_migrating_file (dict_t *dict);
 
-
-/* Acquire non-blocking inodelk on a list of xlators.
- *
- * @lk_array: array of lock requests lock on.
- *
- * @lk_count: number of locks in @lk_array
- *
- * @inodelk_cbk: will be called after inodelk replies are received
- *
- * @retval: -1 if stack_winding inodelk fails. 0 otherwise.
- *          inodelk_cbk is called with appropriate error on errors.
- *          On failure to acquire lock on all members of list, successful
- *          locks are unlocked before invoking cbk.
- */
-
-int
-dht_nonblocking_inodelk (call_frame_t *frame, dht_lock_t **lk_array,
-                         int lk_count, fop_inodelk_cbk_t inodelk_cbk);
-
-/* same as dht_nonblocking_inodelk, but issues sequential blocking locks on
- * @lk_array directly. locks are issued on some order which remains same
- * for a list of xlators (irrespective of order of xlators within list).
- */
-int
-dht_blocking_inodelk (call_frame_t *frame, dht_lock_t **lk_array,
-                      int lk_count, dht_reaction_type_t reaction,
-                      fop_inodelk_cbk_t inodelk_cbk);
-
-int32_t
-dht_unlock_inodelk (call_frame_t *frame, dht_lock_t **lk_array, int lk_count,
-                    fop_inodelk_cbk_t inodelk_cbk);
-
-dht_lock_t *
-dht_lock_new (xlator_t *this, xlator_t *xl, loc_t *loc, short type,
-              const char *domain);
-void
-dht_lock_array_free (dht_lock_t **lk_array, int count);
-
-int32_t
-dht_lock_count (dht_lock_t **lk_array, int lk_count);
-
 int
 dht_layout_sort (dht_layout_t *layout);
 
@@ -1296,5 +1306,4 @@ getChoices (const char *value);
 int
 dht_aggregate_split_brain_xattr (dict_t *dst, char *key, data_t *value);
 
-
 #endif/* _DHT_H */
